package cn.iocoder.yudao.module.member.controller.admin.scoreRule;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.member.convert.scoreRule.ScoreRuleConvert;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import cn.iocoder.yudao.module.member.vo.scoreRule.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;


@Validated
@RestController
@Api(tags = "管理后台 - 积分规则")
@RequestMapping("/member/score-rule")
public class ScoreRuleController {

    @Resource
    private ScoreRuleService scoreRuleService;

    @PostMapping("/create")
    @ApiOperation("创建积分规则")
    @PreAuthorize("@ss.hasPermission('member:score-rule:create')")
    public CommonResult<Long> createScoreRule(@Valid @RequestBody ScoreRuleCreateReqVO createReqVO) {
        return success(scoreRuleService.createScoreRule(createReqVO));
    }

    @PostMapping("/update")
    @ApiOperation("更新积分规则")
    @PreAuthorize("@ss.hasPermission('member:score-rule:update')")
    public CommonResult<Boolean> updateScoreRule(@Valid @RequestBody ScoreRuleUpdateReqVO updateReqVO) {
        scoreRuleService.updateScoreRule(updateReqVO);
        return success(true);
    }

    @PostMapping("/delete")
    @ApiOperation("删除积分规则")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('member:score-rule:delete')")
    public CommonResult<Boolean> deleteScoreRule(@NotNull @RequestBody Long id) {
        scoreRuleService.deleteScoreRule(id);
        return success(true);
    }

    @PostMapping("/get")
    @ApiOperation("获得积分规则详情")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('member:score-rule:query')")
    public CommonResult<ScoreRuleBackVO> getScoreRule(@NotNull @RequestBody Long id) {
        ScoreRuleBackVO scoreRuleBackVO = scoreRuleService.getScoreRule(id);
        return success(scoreRuleBackVO);
    }

    @PostMapping("/list")
    @ApiOperation("获得积分规则列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('member:score-rule:query')")
    public CommonResult<List<ScoreRuleBackVO>> getScoreRuleList(@NotNull @RequestBody Collection<Long> ids) {
        List<ScoreRuleBackVO> list = scoreRuleService.getScoreRuleList(ids);
        return success(list);
    }

    @PostMapping("/page")
    @ApiOperation("获得积分规则分页")
    @PreAuthorize("@ss.hasPermission('member:score-rule:query')")
    public CommonResult<PageResult<ScoreRuleBackVO>> getScoreRulePage(@Valid @RequestBody ScoreRuleQueryVO query, PageVO page) {
        PageResult<ScoreRuleBackVO> pageResult = scoreRuleService.getScoreRulePage(query, page);
        return success(pageResult);
    }
    @PostMapping("/status")
    @ApiOperation("启用关闭")
    @PreAuthorize("@ss.hasPermission('member:score-rule:update')")
    public CommonResult<Boolean> updateStatus(@Valid @RequestBody ScoreRuleStatusReqVO scoreRuleStatusReqVO) {
        Boolean res = scoreRuleService.updateStatus(scoreRuleStatusReqVO);
        return success(res);
    }
    @PostMapping("/copy")
    @ApiOperation("复制规则")
    @PreAuthorize("@ss.hasPermission('member:score-rule:create')")
    public CommonResult<Long> copyScoreRule(@NotNull @RequestBody Long id) {
        Long newId = scoreRuleService.copyScoreRule(id);
        return success(id);
    }
    @PostMapping("/delay")
    @ApiOperation("延期规则")
    @PreAuthorize("@ss.hasPermission('member:score-rule:update')")
    public CommonResult<Boolean> delayScoreRule(@Valid @RequestBody ScoreDelayReqVO scoreDelayReqVO) {
        Boolean res = scoreRuleService.delayScoreRule(scoreDelayReqVO);
        return success(res);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出积分规则Excel")
    @PreAuthorize("@ss.hasPermission('member:score-rule:export')")
    @OperateLog(type = EXPORT)
    public void exportScoreRuleExcel(@Valid ScoreRuleQueryVO query,
              HttpServletResponse response) throws IOException {
        List<ScoreRuleDO> list = scoreRuleService.getScoreRuleList(query);
        // 导出 Excel
        List<ScoreRuleBackVO> datas = ScoreRuleConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "积分规则.xls", "数据", ScoreRuleBackVO.class, datas);
    }
}
