package cn.iocoder.yudao.module.member.service.scoreRule;

import java.time.Instant;
import java.util.*;
import javax.annotation.Resource;

import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.convert.scoreRule.ScoreRuleConvert;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.dal.mysql.scoreRule.ScoreRuleMapper;
import cn.iocoder.yudao.module.member.enums.ScoreRuleStatusEnum;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.vo.scoreRule.*;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRuleOrderVExtraVO;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRuleRegisterExtraVO;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRuleShareExtraVO;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRulerRecommendExtraVO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.*;

/**
 * 积分规则 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class ScoreRuleServiceImpl extends AbstractService<ScoreRuleMapper, ScoreRuleDO> implements ScoreRuleService {

    @Resource
    private ScoreRuleMapper scoreRuleMapper;

    @Override
    public Long createScoreRule(ScoreRuleCreateReqVO createReqVO) {
        ScoreRuleDO scoreRule = ScoreRuleConvert.INSTANCE.convert(createReqVO);
        Integer scoreRuleType = scoreRule.getType();
        setExtraDO(createReqVO, scoreRuleType, scoreRule);
        scoreRuleMapper.insert(scoreRule);
        // 返回
        return scoreRule.getId();
    }


    @Override
    public void updateScoreRule(ScoreRuleUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateScoreRuleExists(updateReqVO.getId());
        ScoreRuleDO scoreRule = ScoreRuleConvert.INSTANCE.convert(updateReqVO);
        setExtraDO(updateReqVO, updateReqVO.getType(), scoreRule);
        // 更新
        ScoreRuleDO updateObj = ScoreRuleConvert.INSTANCE.convert(updateReqVO);
        scoreRuleMapper.updateById(updateObj);
    }

    /**
     * 设置额外信息
     *
     * @param reqVO         请求实体
     * @param scoreRuleType 指标类型
     * @param scoreRule     需要填充额外字段的DO
     */
    private void setExtraDO(ScoreRuleBaseVO reqVO, Integer scoreRuleType, ScoreRuleDO scoreRule) {
        if (scoreRuleType == ScoreRuleTypeEnum.ORDER_V.getValue()) {
            scoreRule.setExtra(JSONUtil.toJsonStr(reqVO.getExtraOrderV()));
        } else if (scoreRuleType == ScoreRuleTypeEnum.REGISTER.getValue()) {
            scoreRule.setExtra(JSONUtil.toJsonStr(reqVO.getExtraRegister()));
        } else if (scoreRuleType == ScoreRuleTypeEnum.RECOMMEND.getValue()) {
            scoreRule.setExtra(JSONUtil.toJsonStr(reqVO.getExtraRecommend()));
        } else if (scoreRuleType == ScoreRuleTypeEnum.SHARE.getValue()) {
            scoreRule.setExtra(JSONUtil.toJsonStr(reqVO.getExtraShare()));
        } else {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
    }

    @Override
    public void deleteScoreRule(Long id) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(id);
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        if (scoreRuleDO.getStatus() == ScoreRuleStatusEnum.ENABLED.getValue()) {
            throw exception(SCORE_RULE_DELETE_ERROR);
        }
        // 删除
        scoreRuleMapper.deleteById(id);
    }

    private void validateScoreRuleExists(Long id) {
        if (scoreRuleMapper.selectById(id) == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
    }

    @Override
    public ScoreRuleBackVO getScoreRule(Long id) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(id);
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        ScoreRuleBackVO scoreRuleBackVO = ScoreRuleConvert.INSTANCE.convert(scoreRuleDO);
        setExtraVO(scoreRuleBackVO);
        return scoreRuleBackVO;
    }


    @Override
    public List<ScoreRuleBackVO> getScoreRuleList(Collection<Long> ids) {
        List<ScoreRuleDO> scoreRuleDOS = scoreRuleMapper.selectBatchIds(ids);
        List<ScoreRuleBackVO> scoreRuleBackVOS = ScoreRuleConvert.INSTANCE.convertList(scoreRuleDOS);
        scoreRuleBackVOS.forEach(this::setExtraVO);
        return scoreRuleBackVOS;
    }

    @Override
    public PageResult<ScoreRuleBackVO> getScoreRulePage(ScoreRuleQueryVO query, PageVO page) {
        PageResult<ScoreRuleDO> pageResult = scoreRuleMapper.selectPage(page, query);
        PageResult<ScoreRuleBackVO> scoreRuleBackVOPageResult = ScoreRuleConvert.INSTANCE.convertPage(pageResult);
        scoreRuleBackVOPageResult.getList().forEach(this::setExtraVO);
        return scoreRuleBackVOPageResult;
    }

    /**
     * 设置返回VO额外信息
     *
     * @param scoreRuleBackVO VO
     */
    private void setExtraVO(ScoreRuleBackVO scoreRuleBackVO) {
        if (scoreRuleBackVO.getType() == ScoreRuleTypeEnum.ORDER_V.getValue()) {
            scoreRuleBackVO.setExtraOrderV(JSONUtil.toBean(scoreRuleBackVO.getExtra(), ScoreRuleOrderVExtraVO.class));
        } else if (scoreRuleBackVO.getType() == ScoreRuleTypeEnum.REGISTER.getValue()) {
            scoreRuleBackVO.setExtraRegister(JSONUtil.toBean(scoreRuleBackVO.getExtra(), ScoreRuleRegisterExtraVO.class));
        } else if (scoreRuleBackVO.getType() == ScoreRuleTypeEnum.RECOMMEND.getValue()) {
            scoreRuleBackVO.setExtraRecommend(JSONUtil.toBean(scoreRuleBackVO.getExtra(), ScoreRulerRecommendExtraVO.class));
        } else if (scoreRuleBackVO.getType() == ScoreRuleTypeEnum.SHARE.getValue()) {
            scoreRuleBackVO.setExtraShare(JSONUtil.toBean(scoreRuleBackVO.getExtra(), ScoreRuleShareExtraVO.class));
        } else {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
    }

    @Override
    public List<ScoreRuleDO> getScoreRuleList(ScoreRuleQueryVO query) {
        return scoreRuleMapper.selectList(query);
    }

    @Override
    public Boolean updateStatus(ScoreRuleStatusReqVO scoreRuleStatusReqVO) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(scoreRuleStatusReqVO.getId());
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        ScoreRuleDO upScoreRuleDO = new ScoreRuleDO();
        upScoreRuleDO.setId(scoreRuleStatusReqVO.getId());
        upScoreRuleDO.setStatus(scoreRuleStatusReqVO.getStatus());
        int updated = scoreRuleMapper.updateById(upScoreRuleDO);
        return updated > 0;
    }

    @Override
    public Long copyScoreRule(Long id) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(id);
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        scoreRuleDO.setStatus(ScoreRuleStatusEnum.DISABLED.getValue());
        scoreRuleDO.setId(null);
        scoreRuleDO.setCreateTime(null);
        scoreRuleDO.setUpdateTime(null);
        scoreRuleMapper.insert(scoreRuleDO);
        return scoreRuleDO.getId();
    }

    @Override
    public Boolean delayScoreRule(ScoreDelayReqVO delayReqVO) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(delayReqVO.getId());
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        if (scoreRuleDO.getStatus() != ScoreRuleStatusEnum.ENABLED.getValue()) {
            throw exception(SCORE_RULE_UPDATE_ERROR);
        }
        Instant now = Instant.now();
        //结束时间不能小于当前时间
        if (!delayReqVO.getEndTime().toInstant().isAfter(now)) {
            throw exception(SCORE_RULE_UPDATE_ERROR);
        }
        ScoreRuleDO upScoreRuleDO = new ScoreRuleDO();
        upScoreRuleDO.setId(delayReqVO.getId());
        upScoreRuleDO.setEndTime(delayReqVO.getEndTime());
        int updated = scoreRuleMapper.updateById(upScoreRuleDO);
        return updated > 0;
    }
}
